@@ -204,94 +204,31 @@ const char* ecs_cpp_trim_module(
     return type_name;
 }
 
-// Validate registered component
-void ecs_cpp_component_validate(
+ecs_entity_t ecs_cpp_component_find(
     ecs_world_t *world,
     ecs_entity_t id,
     const char *name,
     const char *symbol,
     size_t size,
     size_t alignment,
-    bool implicit_name)
-{
-    /* If entity has a name check if it matches */
-    if (ecs_is_valid(world, id) && ecs_get_name(world, id) != NULL) {
-        if (!implicit_name && id >= EcsFirstUserComponentId) {
-#ifndef FLECS_NDEBUG
-            char *path = ecs_get_path_w_sep(
-                world, 0, id, "::", NULL);
-            if (ecs_os_strcmp(path, name)) {
-                ecs_abort(ECS_INCONSISTENT_NAME,
-                    "component '%s' already registered with name '%s'",
-                    name, path);
-            }
-            ecs_os_free(path);
-#endif
-        }
-
-        if (symbol) {
-            const char *existing_symbol = ecs_get_symbol(world, id);
-            if (existing_symbol) {
-                if (ecs_os_strcmp(symbol, existing_symbol)) {
-                    ecs_abort(ECS_INCONSISTENT_NAME,
-                        "component '%s' with symbol '%s' already registered with symbol '%s'",
-                        name, symbol, existing_symbol);
-                }
-            }
-        }
-    } else {
-        /* Ensure that the entity id valid */
-        if (!ecs_is_alive(world, id)) {
-            ecs_make_alive(world, id);
-        }
-
-        /* Register name with entity, so that when the entity is created the
-        * correct id will be resolved from the name. Only do this when the
-        * entity is empty. */
-        ecs_add_path_w_sep(world, id, 0, name, "::", "::");
-    }
-
-    /* If a component was already registered with this id but with a 
-     * different size, the ecs_component_init function will fail. */
-
-    /* We need to explicitly call ecs_component_init here again. Even though
-     * the component was already registered, it may have been registered
-     * with a different world. This ensures that the component is registered
-     * with the same id for the current world. 
-     * If the component was registered already, nothing will change. */
-    ecs_entity_t ent = ecs_component_init(world, &(ecs_component_desc_t){
-        .entity = id,
-        .type.size = flecs_uto(int32_t, size),
-        .type.alignment = flecs_uto(int32_t, alignment)
-    });
-    (void)ent;
-    ecs_assert(ent == id, ECS_INTERNAL_ERROR, NULL);
-}
-
-ecs_entity_t ecs_cpp_component_register(
-    ecs_world_t *world,
-    ecs_entity_t id,
-    const char *name,
-    const char *symbol,
-    ecs_size_t size,
-    ecs_size_t alignment,
     bool implicit_name,
     bool *existing_out)
 {
     (void)size;
     (void)alignment;
 
+    ecs_assert(existing_out != NULL, ECS_INTERNAL_ERROR, NULL);
+
     /* If the component is not yet registered, ensure no other component
      * or entity has been registered with this name. Ensure component is 
      * looked up from root. */
-    bool existing = false;
     ecs_entity_t prev_scope = ecs_set_scope(world, 0);
     ecs_entity_t ent;
     if (id) {
         ent = id;
     } else {
         ent = ecs_lookup_path_w_sep(world, 0, name, "::", "::", false);
-        existing = ent != 0 && ecs_has(world, ent, EcsComponent);
+        *existing_out = ent != 0 && ecs_has(world, ent, EcsComponent);
     }
     ecs_set_scope(world, prev_scope);
 
@@ -319,16 +256,23 @@ ecs_entity_t ecs_cpp_component_register(
                  * to alias the type, vs. accidentally registering an unrelated
                  * type with the same size/alignment. */
                 char *type_path = ecs_get_path(world, ent);
-                if (ecs_os_strcmp(type_path, symbol) || 
-                    component->size != size || 
-                    component->alignment != alignment) 
-                {
+                if (ecs_os_strcmp(type_path, symbol)) {
                     ecs_err(
                         "component with name '%s' is already registered for"\
                         " type '%s' (trying to register for type '%s')",
                             name, sym, symbol);
                     ecs_abort(ECS_NAME_IN_USE, NULL);
                 }
+
+                if (flecs_itosize(component->size) != size || 
+                    flecs_itosize(component->alignment) != alignment)
+                {
+                    ecs_err(
+                        "component with name '%s' is already registered with"\
+                        " mismatching size/alignment)", name);
+                    ecs_abort(ECS_INVALID_COMPONENT_SIZE, NULL);
+                }
+
                 ecs_os_free(type_path);
             } else if (!sym) {
                 ecs_set_symbol(world, ent, symbol);
@@ -339,17 +283,14 @@ ecs_entity_t ecs_cpp_component_register(
      * registered under a different name. */
     } else if (!implicit_name) {
         ent = ecs_lookup_symbol(world, symbol, false, false);
-        ecs_assert(ent == 0 || (ent == id), ECS_INCONSISTENT_COMPONENT_ID, symbol);
-    }
-
-    if (existing_out) {
-        *existing_out = existing;
+        ecs_assert(ent == 0 || (ent == id), 
+            ECS_INCONSISTENT_COMPONENT_ID, symbol);
     }
 
     return ent;
 }
 
-ecs_entity_t ecs_cpp_component_register_explicit(
+ecs_entity_t ecs_cpp_component_register(
     ecs_world_t *world,
     ecs_entity_t s_id,
     ecs_entity_t id,
@@ -362,7 +303,8 @@ ecs_entity_t ecs_cpp_component_register_explicit(
     bool *existing_out)
 {
     char *existing_name = NULL;
-    if (existing_out) *existing_out = false;
+
+    ecs_assert(existing_out != NULL, ECS_INTERNAL_ERROR, NULL);
 
     // If an explicit id is provided, it is possible that the symbol and
     // name differ from the actual type, as the application may alias
@@ -375,7 +317,7 @@ ecs_entity_t ecs_cpp_component_register_explicit(
             if (id) {
                 existing_name = ecs_get_path_w_sep(world, 0, id, "::", "::");
                 name = existing_name;
-                if (existing_out) *existing_out = true;
+                *existing_out = true;
             } else {
                 // If type is not yet known, derive from type name
                 name = ecs_cpp_trim_module(world, type_name);
@@ -429,14 +371,16 @@ ecs_entity_t ecs_cpp_component_register_explicit(
 
 void ecs_cpp_enum_init(
     ecs_world_t *world,
-    ecs_entity_t id)
+    ecs_entity_t id,
+    ecs_entity_t underlying_type)
 {
     (void)world;
     (void)id;
+    (void)underlying_type;
 #ifdef FLECS_META
     ecs_suspend_readonly_state_t readonly_state;
     world = flecs_suspend_readonly(world, &readonly_state);
-    ecs_set(world, id, EcsEnum, {0});
+    ecs_set(world, id, EcsEnum, { .underlying_type = underlying_type });
     flecs_resume_readonly(world, &readonly_state);
 #endif
 }
@@ -446,8 +390,11 @@ ecs_entity_t ecs_cpp_enum_constant_register(
     ecs_entity_t parent,
     ecs_entity_t id,
     const char *name,
-    int value)
+    void *value,
+    ecs_entity_t value_type,
+    size_t value_size)
 {
+#ifdef FLECS_META
     ecs_suspend_readonly_state_t readonly_state;
     world = flecs_suspend_readonly(world, &readonly_state);
 
@@ -468,34 +415,38 @@ ecs_entity_t ecs_cpp_enum_constant_register(
     ecs_assert(id != 0, ECS_INVALID_OPERATION, name);
     ecs_set_scope(world, prev);
 
-    #ifdef FLECS_DEBUG
+#ifdef FLECS_DEBUG
     const EcsComponent *cptr = ecs_get(world, parent, EcsComponent);
     ecs_assert(cptr != NULL, ECS_INVALID_PARAMETER, "enum is not a component");
-    ecs_assert(cptr->size == ECS_SIZEOF(int32_t), ECS_UNSUPPORTED,
-        "enum component must have 32bit size");
-    #endif
-
-#ifdef FLECS_META
-    ecs_set_id(world, id, ecs_pair(EcsConstant, ecs_id(ecs_i32_t)), 
-        sizeof(ecs_i32_t), &value);
 #endif
 
+    ecs_set_id(world, id, ecs_pair(EcsConstant, value_type), value_size, value);
+
     flecs_resume_readonly(world, &readonly_state);
 
-    ecs_trace("#[green]constant#[reset] %s.%s created with value %d", 
-        ecs_get_name(world, parent), name, value);
+    if (ecs_should_log(0)) {
+        ecs_value_t v = { .type = value_type, .ptr = value };
+        char *str = NULL;
+        ecs_meta_cursor_t cur = ecs_meta_cursor(world, 
+            ecs_id(ecs_string_t), &str);
+        ecs_meta_set_value(&cur, &v);
+        ecs_trace("#[green]constant#[reset] %s.%s created with value %s", 
+            ecs_get_name(world, parent), name, str);
+        ecs_os_free(str);
+    }
 
     return id;
-}
-
-static int32_t flecs_reset_count = 0;
-
-int32_t ecs_cpp_reset_count_get(void) {
-    return flecs_reset_count;
-}
-
-int32_t ecs_cpp_reset_count_inc(void) {
-    return ++flecs_reset_count;
+#else
+    (void)world;
+    (void)parent;
+    (void)id;
+    (void)name;
+    (void)value;
+    (void)value_type;
+    (void)value_size;
+    ecs_err("enum reflection not supported without FLECS_META addon");
+    return 0;
+#endif
 }
 
 #ifdef FLECS_META
