@@ -80,7 +80,7 @@ const char* flecs_script_comma_expr(
                 if (is_base_list) {
                     flecs_script_insert_pair_tag(parser, "IsA", Token(0));
                 } else {
-                    flecs_script_insert_entity(parser, Token(0));
+                    flecs_script_insert_entity(parser, Token(0), false);
                 }
 
                 LookAhead_1(',',
@@ -111,11 +111,11 @@ const char* flecs_script_with_expr(
                 pos = lookahead;
 
                 // Position ( expr )
-                Expr(')',
+                Initializer(')',
                     ecs_script_component_t *component =
                         flecs_script_insert_component(parser, Token(0));
                     component->node.kind = EcsAstWithComponent;
-                    component->expr = Token(2);
+                    component->expr = INITIALIZER;
                     EndOfRule;
                 )
             )
@@ -142,12 +142,12 @@ const char* flecs_script_with_expr(
                     pos = lookahead;
 
                     // (Eats, Apples) ( expr )
-                    Expr(')',
+                    Initializer(')',
                         ecs_script_component_t *component =
                             flecs_script_insert_pair_component(parser, 
                                 Token(1), Token(3));
                         component->node.kind = EcsAstWithComponent;
-                        component->expr = Token(6);
+                        component->expr = INITIALIZER;
                         EndOfRule;
                     )
                 )
@@ -205,13 +205,13 @@ const char* flecs_script_paren_expr(
 {
     ParserBegin;
 
-    Expr(')',
+    Initializer(')',
         entity->kind_w_expr = true;
 
         Scope(entity->scope, 
             ecs_script_component_t *component = 
                 flecs_script_insert_component(parser, kind);
-            component->expr = Token(0);
+            component->expr = INITIALIZER;
         )
 
         Parse(
@@ -230,6 +230,123 @@ const char* flecs_script_paren_expr(
     ParserEnd;
 }
 
+/* Parse a single statement */
+static
+const char* flecs_script_if_stmt(
+    ecs_script_parser_t *parser,
+    const char *pos)
+{
+    ParserBegin;
+
+    // if expr
+    Expr('\0',
+        // if expr {
+        Parse_1('{', {
+            ecs_script_if_t *stmt = flecs_script_insert_if(parser);
+            stmt->expr = EXPR;
+            pos = flecs_script_scope(parser, stmt->if_true, pos);
+            if (!pos) {
+                goto error;
+            }
+
+            // if expr { } else
+            LookAhead_1(EcsTokKeywordElse, 
+                pos = lookahead;
+
+                Parse(
+                    // if expr { } else if
+                    case EcsTokKeywordIf: {
+                        Scope(stmt->if_false, 
+                            return flecs_script_if_stmt(parser, pos);
+                        )
+                    }
+
+                    // if expr { } else\n if
+                    case EcsTokNewline: {
+                        Parse_1(EcsTokKeywordIf,
+                            Scope(stmt->if_false, 
+                                return flecs_script_if_stmt(parser, pos);
+                            )
+                        )
+                    }
+
+                    // if expr { } else {
+                    case '{': {
+                        return flecs_script_scope(parser, stmt->if_false, pos);
+                    }
+                )
+            )
+
+            EndOfRule;
+        });
+    )
+
+    ParserEnd;
+}
+
+static
+const char* flecs_script_parse_var(
+    ecs_script_parser_t *parser,
+    const char *pos,
+    ecs_script_tokenizer_t *tokenizer,
+    bool is_prop)
+{
+    Parse_1(EcsTokIdentifier,
+        ecs_script_var_node_t *var = flecs_script_insert_var(
+            parser, Token(1));
+        var->node.kind = is_prop ? EcsAstProp : EcsAstConst;
+
+        Parse(
+            // const color =
+            case '=': {
+                // const color = Color :
+                LookAhead_2(EcsTokIdentifier, ':',
+                    pos = lookahead;
+
+                    var->type = Token(3);
+
+                    // const color = Color: {
+                    LookAhead_1('{',
+                        // const color = Color: {expr}
+                        pos = lookahead;
+                        Initializer('}',
+                            var->expr = INITIALIZER;
+                            EndOfRule;
+                        )
+                    )
+
+                    // const color = Color: expr\n
+                    Initializer('\n',
+                        var->expr = INITIALIZER;
+                        EndOfRule;
+                    )
+                )
+
+                // const PI = expr\n
+                Expr('\n',
+                    Warning("'%s var = expr' syntax is deprecated"
+                        ", use '%s var: expr' instead", 
+                            is_prop ? "prop" : "const",
+                            is_prop ? "prop" : "const");
+                    var->expr = EXPR;
+                    EndOfRule;
+                )
+            }
+
+            case ':': {
+                // const PI: expr\n
+                Expr('\n',
+                    var->expr = EXPR;
+                    EndOfRule;
+                )
+            }
+        )
+    )
+
+error:
+    return NULL;
+}
+
 /* Parse a single statement */
 static
 const char* flecs_script_stmt(
@@ -238,10 +355,13 @@ const char* flecs_script_stmt(
 {
     ParserBegin;
 
+    bool name_is_expr_0 = false;
+    bool name_is_expr_1 = false;
+
     Parse(
         case EcsTokIdentifier:        goto identifier;
-        case '{':                     return flecs_script_scope(parser, 
-                                        flecs_script_insert_scope(parser), pos);
+        case EcsTokString:            goto string_name;
+        case '{':                     goto anonymous_entity;
         case '(':                     goto paren;
         case '@':                     goto annotation;
         case EcsTokKeywordWith:       goto with_stmt;
@@ -251,9 +371,24 @@ const char* flecs_script_stmt(
         case EcsTokKeywordProp:       goto prop_var;
         case EcsTokKeywordConst:      goto const_var;
         case EcsTokKeywordIf:         goto if_stmt;
+        case EcsTokKeywordFor:        goto for_stmt;
         EcsTokEndOfStatement:         EndOfRule;
     );
 
+anonymous_entity: {
+    return flecs_script_scope(parser, 
+        flecs_script_insert_entity(parser, "_", false)->scope, pos);
+}
+
+string_name:
+    /* If this is an interpolated string, we need to evaluate it as expression
+     * at evaluation time. Otherwise we can just use the string as name. The 
+     * latter is useful if an entity name contains special characters that are
+     * not allowed in identifier tokens. */
+    if (flecs_string_is_interpolated(Token(0))) {
+        name_is_expr_0 = true;
+    }
+
 identifier: {
     // enterprise } (end of scope)
     LookAhead_1('}',
@@ -265,12 +400,16 @@ identifier: {
         case '{': {
             return flecs_script_scope(parser, 
                 flecs_script_insert_entity(
-                    parser, Token(0))->scope, pos);
+                    parser, Token(0), name_is_expr_0)->scope, pos);
         }
 
         // Red,
         case ',': {
-            flecs_script_insert_entity(parser, Token(0));
+            if (name_is_expr_0) {
+                Error("expression not allowed as entity name here");
+            }
+
+            flecs_script_insert_entity(parser, Token(0), false);
             pos = flecs_script_comma_expr(parser, pos, false);
             EndOfRule;
         }
@@ -282,7 +421,7 @@ identifier: {
                 pos = lookahead;
                 return flecs_script_scope(parser, 
                     flecs_script_insert_entity(
-                        parser, Token(0))->scope, pos);
+                        parser, Token(0), name_is_expr_0)->scope, pos);
             )
 
             goto insert_tag;
@@ -312,6 +451,11 @@ identifier: {
         case EcsTokIdentifier: {
             goto identifier_identifier;
         }
+
+        // Spaceship "enterprise"
+        case EcsTokString: {
+            goto identifier_string;
+        }
     )
 }
 
@@ -320,13 +464,13 @@ insert_tag: {
         if (!flecs_script_insert_var_component(parser, &Token(0)[1])) {
             Error(
                 "invalid context for variable component '%s': must be "
-                    "part of entity", tokens[0].value);
+                    "part of entity", tokenizer->tokens[0].value);
         }
     } else {
         if (!flecs_script_insert_tag(parser, Token(0))) {
             Error(
                 "invalid context for tag '%s': must be part of entity", 
-                tokens[0].value);
+                tokenizer->tokens[0].value);
         }
     }
 
@@ -396,97 +540,43 @@ template_stmt: {
 // prop
 prop_var: {
     // prop color = Color:
-    Parse_4(EcsTokIdentifier, '=', EcsTokIdentifier, ':',
-        ecs_script_var_node_t *var = flecs_script_insert_var(
-            parser, Token(1));
-        var->node.kind = EcsAstProp;
-        var->type = Token(3);
-
-        // prop color = Color : {
-        LookAhead_1('{',
-            // prop color = Color: {expr}
-            pos = lookahead;
-            Expr('}',
-                var->expr = Token(6);
-                EndOfRule;
-            )
-        )
-
-        // prop color = Color : expr\n
-        Expr('\n',
-            var->expr = Token(5);
-            EndOfRule;
-        )
-    )
+    return flecs_script_parse_var(parser, pos, tokenizer, true);
 }
 
 // const
 const_var: {
     // const color
-    Parse_1(EcsTokIdentifier,
-        ecs_script_var_node_t *var = flecs_script_insert_var(
-            parser, Token(1));
-        var->node.kind = EcsAstConst;
-
-        Parse(
-            // const color =
-            case '=': {
-                // const color = Color :
-                LookAhead_2(EcsTokIdentifier, ':',
-                    pos = lookahead;
-
-                    var->type = Token(3);
-
-                    // const color = Color: {
-                    LookAhead_1('{',
-                        // const color = Color: {expr}
-                        pos = lookahead;
-                        Expr('}',
-                            var->expr = Token(6);
-                            EndOfRule;
-                        )
-                    )
-
-                    // const color = Color: expr\n
-                    Expr('\n',
-                        var->expr = Token(5);
-                        EndOfRule;
-                    )
-                )
-
-                // const PI = expr\n
-                Expr('\n',
-                    var->expr = Token(3);
-                    EndOfRule;
-                )
-            }
-        )
-    )
+    return flecs_script_parse_var(parser, pos, tokenizer, false);
 }
 
 // if
 if_stmt: {
-    // if expr {
-    Expr('{',
-        ecs_script_if_t *stmt = flecs_script_insert_if(parser);
-        stmt->expr = Token(1);
-        pos = flecs_script_scope(parser, stmt->if_true, pos);
-        if (!pos) {
-            goto error;
-        }
+    return flecs_script_if_stmt(parser, pos);
+}
 
-        // if expr { } else
-        LookAhead_1(EcsTokKeywordElse, 
-            pos = lookahead;
+// for
+for_stmt: {
+    // for i
+    Parse_2(EcsTokIdentifier, EcsTokKeywordIn, {
+        Expr(0, {
+            ecs_expr_node_t *from = EXPR;
+            Parse_1(EcsTokRange, {
+                Expr(0, {
+                    ecs_expr_node_t *to = EXPR;
+                    ecs_script_for_range_t *stmt = 
+                        flecs_script_insert_for_range(parser);
+                    stmt->loop_var = Token(1);
+                    stmt->from = from;
+                    stmt->to = to;
 
-            // if expr { } else {
-            Parse_1('{',
-                return flecs_script_scope(parser, stmt->if_false, pos);
-            )
-        )
+                    Parse_1('{', {
+                        return flecs_script_scope(parser, stmt->scope, pos);
+                    });
+                });
+            });
+        });
 
-        EndOfRule;
-    )
+    });
 }
 
 // (
@@ -514,16 +604,29 @@ pair: {
 
         // (Eats, Apples):
         case ':': {
-            // (Eats, Apples): {
-            Parse_1('{',
-                // (Eats, Apples): { expr }
-                Expr('}',
+            // Use lookahead so that expression parser starts at "match"
+            LookAhead_1(EcsTokKeywordMatch, {
+                // (Eats, Apples): match expr
+                Expr('\n', {
                     ecs_script_component_t *comp = 
                         flecs_script_insert_pair_component(
                             parser, Token(1), Token(3));
-                    comp->expr = Token(7);
-                    EndOfRule;
-                )
+                    comp->expr = EXPR;
+                    EndOfRule; 
+                })
+            })
+
+            // (Eats, Apples): {
+            Parse_1('{', {
+                    // (Eats, Apples): { expr }
+                    Initializer('}',
+                        ecs_script_component_t *comp = 
+                            flecs_script_insert_pair_component(
+                                parser, Token(1), Token(3));
+                        comp->expr = INITIALIZER;
+                        EndOfRule;
+                    )
+                }
             )
         }
 
@@ -568,7 +671,7 @@ identifier_flag: {
                                 ecs_script_component_t *comp = 
                                     flecs_script_insert_pair_component(
                                         parser, Token(3), Token(5));
-                                comp->expr = Token(9);
+                                comp->expr = EXPR;
                                 EndOfRule; 
                             })
                         )
@@ -593,9 +696,10 @@ identifier_flag: {
                     Parse_1('{',
                         // auto_override | Position: {expr}
                         Expr('}', {
-                            ecs_script_component_t *comp = flecs_script_insert_component(
-                                parser, Token(2));
-                            comp->expr = Token(5);
+                            ecs_script_component_t *comp = 
+                                flecs_script_insert_component(
+                                    parser, Token(2));
+                            comp->expr = EXPR;
                             EndOfRule; 
                         })
                     )
@@ -623,10 +727,17 @@ identifier_colon: {
         )
     }
 
+    {
+        // Position: match
+        LookAhead_1(EcsTokKeywordMatch,
+            goto component_expr_match;
+        )
+    }
+
     // enterprise : SpaceShip
     Parse_1(EcsTokIdentifier, {
         ecs_script_entity_t *entity = flecs_script_insert_entity(
-            parser, Token(0));
+            parser, Token(0), name_is_expr_0);
 
         Scope(entity->scope, 
             flecs_script_insert_pair_tag(parser, "IsA", Token(2));
@@ -652,20 +763,32 @@ identifier_colon: {
 // x =
 identifier_assign: {
     ecs_script_entity_t *entity = flecs_script_insert_entity(
-        parser, Token(0));
+        parser, Token(0), name_is_expr_0);
 
     // x = Position:
     LookAhead_2(EcsTokIdentifier, ':',
         pos = lookahead;
 
+        // Use lookahead so that expression parser starts at "match"
+        LookAhead_1(EcsTokKeywordMatch, {
+            // (Eats, Apples): match expr
+            Expr('\n', {
+                ecs_script_component_t *comp = 
+                    flecs_script_insert_pair_component(
+                        parser, Token(1), Token(3));
+                comp->expr = EXPR;
+                EndOfRule; 
+            })
+        })
+
         // x = Position: {
         Parse_1('{', {
             // x = Position: {expr}
-            Expr('}', 
+            Expr('}',
                 Scope(entity->scope, 
                     ecs_script_component_t *comp = 
                         flecs_script_insert_component(parser, Token(2));
-                    comp->expr = Token(5);
+                    comp->expr = EXPR;
                 )
 
                 // x = Position: {expr}\n
@@ -678,11 +801,11 @@ identifier_assign: {
     )
 
     // x = f32\n
-    Expr('\n',
+    Initializer('\n',
         Scope(entity->scope, 
             ecs_script_default_component_t *comp = 
                 flecs_script_insert_default_component(parser);
-            comp->expr = Token(2);
+            comp->expr = INITIALIZER;
         )
         
         EndOfRule;
@@ -690,9 +813,15 @@ identifier_assign: {
 }
 
 // Spaceship enterprise
+identifier_string: {
+    if (flecs_string_is_interpolated(Token(1))) {
+        name_is_expr_1 = true;
+    }
+}
+
 identifier_identifier: {
     ecs_script_entity_t *entity = flecs_script_insert_entity(
-        parser, Token(1));
+        parser, Token(1), name_is_expr_1);
     entity->kind = Token(0);
 
     // Spaceship enterprise :
@@ -735,17 +864,17 @@ identifier_identifier_x:
 // SpaceShip(
 identifier_paren: {
     // SpaceShip()
-    Expr(')', 
+    Initializer(')',
         Parse(
             // SpaceShip(expr)\n
             EcsTokEndOfStatement: {
                 ecs_script_entity_t *entity = flecs_script_insert_entity(
-                    parser, NULL);
+                    parser, NULL, false);
 
                 Scope(entity->scope, 
                     ecs_script_component_t *comp = 
                         flecs_script_insert_component(parser, Token(0)); 
-                    comp->expr = Token(2);
+                    comp->expr = INITIALIZER;
                 )
 
                 EndOfRule;
@@ -754,12 +883,12 @@ identifier_paren: {
             // SpaceShip(expr) {
             case '{': {
                 ecs_script_entity_t *entity = flecs_script_insert_entity(
-                    parser, NULL);
+                    parser, NULL, false);
 
                 Scope(entity->scope, 
                     ecs_script_component_t *comp = 
                         flecs_script_insert_component(parser, Token(0)); 
-                    comp->expr = Token(2);
+                    comp->expr = INITIALIZER;
                 )
 
                 return flecs_script_scope(parser, entity->scope, pos);
@@ -775,7 +904,7 @@ component_expr_scope: {
     Expr('}', {
         ecs_script_component_t *comp = flecs_script_insert_component(
             parser, Token(0));
-        comp->expr = Token(3);
+        comp->expr = EXPR;
         EndOfRule; 
     })
 }
@@ -786,12 +915,24 @@ component_expr_collection: {
     Expr(']', {
         ecs_script_component_t *comp = flecs_script_insert_component(
             parser, Token(0));
-        comp->expr = Token(3);
+        comp->expr = EXPR;
         comp->is_collection = true;
         EndOfRule; 
     })
 }
 
+// Position: match
+component_expr_match: {
+
+    // Position: match expr
+    Expr('\n', {
+        ecs_script_component_t *comp = flecs_script_insert_component(
+            parser, Token(0));
+        comp->expr = EXPR;
+        EndOfRule; 
+    })
+}
+
     ParserEnd;
 }
 
@@ -799,8 +940,11 @@ component_expr_collection: {
 ecs_script_t* ecs_script_parse(
     ecs_world_t *world,
     const char *name,
-    const char *code) 
+    const char *code,
+    const ecs_script_eval_desc_t *desc) 
 {
+    (void)desc; /* Will be used in future to expand type checking features */
+
     if (!code) {
         code = "";
     }
@@ -824,8 +968,8 @@ ecs_script_t* ecs_script_parse(
      * ensures that AST nodes don't need to do separate allocations for the data
      * they contain. */
     impl->token_buffer_size = ecs_os_strlen(code) * 2 + 1;
-    impl->token_buffer = flecs_alloc(
-        &impl->allocator, impl->token_buffer_size);
+    impl->token_buffer = flecs_alloc_w_dbg_info(
+        &impl->allocator, impl->token_buffer_size, "token buffer");
     parser.token_cur = impl->token_buffer;
 
     /* Start parsing code */
@@ -844,6 +988,8 @@ ecs_script_t* ecs_script_parse(
         }
     } while (true);
 
+    impl->token_remaining = parser.token_cur;
+
     return script;
 error:
     ecs_script_free(script);
