@@ -63,6 +63,7 @@ void Sparse_count_of_null(void);
 void Sparse_try_low_after_ensure_high(void);
 void Sparse_is_alive_low_after_ensure_high(void);
 void Sparse_remove_low_after_ensure_high(void);
+void Sparse_ensure_skip_generation(void);
 
 // Testsuite 'Strbuf'
 void Strbuf_setup(void);
@@ -102,6 +103,10 @@ void Strbuf_append_inf(void);
 void Strbuf_append_nan_delim(void);
 void Strbuf_append_inf_delim(void);
 
+// Testsuite 'Allocator'
+void Allocator_setup(void);
+void Allocator_init_fini_empty(void);
+
 bake_test_case Map_testcases[] = {
     {
         "count",
@@ -305,6 +310,10 @@ bake_test_case Sparse_testcases[] = {
     {
         "remove_low_after_ensure_high",
         Sparse_remove_low_after_ensure_high
+    },
+    {
+        "ensure_skip_generation",
+        Sparse_ensure_skip_generation
     }
 };
 
@@ -451,6 +460,14 @@ bake_test_case Strbuf_testcases[] = {
     }
 };
 
+bake_test_case Allocator_testcases[] = {
+    {
+        "init_fini_empty",
+        Allocator_init_fini_empty
+    }
+};
+
+
 static bake_test_suite suites[] = {
     {
         "Map",
@@ -463,7 +480,7 @@ static bake_test_suite suites[] = {
         "Sparse",
         Sparse_setup,
         NULL,
-        21,
+        22,
         Sparse_testcases
     },
     {
@@ -472,9 +489,16 @@ static bake_test_suite suites[] = {
         NULL,
         35,
         Strbuf_testcases
+    },
+    {
+        "Allocator",
+        Allocator_setup,
+        NULL,
+        1,
+        Allocator_testcases
     }
 };
 
 int main(int argc, char *argv[]) {
-    return bake_test_run("collections", argc, argv, suites, 3);
+    return bake_test_run("collections", argc, argv, suites, 4);
 }
