@@ -61,6 +61,9 @@ typedef struct {
 
 void install_test_abort(void);
 
+#define test_json(v1, v2) _test_json(v1, v2, #v1, #v2, __FILE__, __LINE__)
+void _test_json(const char* str1, const char *str2, const char* vstr1, const char *vstr2, const char *file, int line);
+
 #ifdef __cplusplus
 }
 #endif
