@@ -51,7 +51,7 @@ void Units_member_w_unit_type(void) {
     });
     test_assert(u != 0);
     test_str("percentage", ecs_get_name(world, u));
-    test_assert(ecs_has(world, u, EcsMetaType));
+    test_assert(ecs_has(world, u, EcsType));
 
     ecs_unit_init(world, &(ecs_unit_desc_t){
         .entity = u,
@@ -106,7 +106,7 @@ void Units_cursor_get_unit(void) {
         }
     });
 
-    ecs_entity_t e = ecs_new_id(world);
+    ecs_entity_t e = ecs_new(world);
     void *ptr = ecs_ensure_id(world, e, s);
     test_assert(ptr != NULL);
 
@@ -139,7 +139,7 @@ void Units_cursor_get_unit_type(void) {
         }
     });
 
-    ecs_entity_t e = ecs_new_id(world);
+    ecs_entity_t e = ecs_new(world);
     void *ptr = ecs_ensure_id(world, e, s);
     test_assert(ptr != NULL);
 
@@ -351,7 +351,7 @@ void Units_unit_w_over(void) {
 void Units_member_w_invalid_unit(void) {
     ecs_world_t *world = ecs_init();
 
-    ecs_entity_t u = ecs_new_id(world); /* not a unit */
+    ecs_entity_t u = ecs_new(world); /* not a unit */
     test_assert(u != 0);
 
     ecs_log_set_level(-4);
@@ -369,7 +369,7 @@ void Units_member_w_invalid_unit(void) {
 void Units_unit_w_invalid_quantity(void) {
     ecs_world_t *world = ecs_init();
 
-    ecs_entity_t q = ecs_new_id(world); /* not a quantity */
+    ecs_entity_t q = ecs_new(world); /* not a quantity */
     test_assert(q != 0);
 
     ecs_log_set_level(-4);
@@ -387,7 +387,7 @@ void Units_unit_w_invalid_quantity(void) {
 void Units_unit_w_invalid_derived(void) {
     ecs_world_t *world = ecs_init();
 
-    ecs_entity_t u_1 = ecs_new_id(world); /* not a unit */
+    ecs_entity_t u_1 = ecs_new(world); /* not a unit */
     test_assert(u_1 != 0);
 
     ecs_log_set_level(-4);
@@ -405,7 +405,7 @@ void Units_unit_w_invalid_derived(void) {
 void Units_unit_w_invalid_over(void) {
     ecs_world_t *world = ecs_init();
 
-    ecs_entity_t u_1 = ecs_new_id(world); /* not a unit */
+    ecs_entity_t u_1 = ecs_new(world); /* not a unit */
     test_assert(u_1 != 0);
 
     ecs_entity_t u_2 = ecs_unit_init(world, &(ecs_unit_desc_t){
@@ -631,11 +631,11 @@ void Units_set_unit(void) {
     ecs_world_t *world = ecs_init();
 
     ecs_entity_t q = ecs_entity_init(world, &(ecs_entity_desc_t){
-        .add = { EcsQuantity }
+        .add = ecs_ids( EcsQuantity )
     });
     test_assert(q != 0);
 
-    ecs_entity_t u = ecs_new_id(world);
+    ecs_entity_t u = ecs_new(world);
     test_assert(u != 0);
 
     ecs_set(world, u, EcsUnit, {
@@ -679,18 +679,18 @@ void Units_set_unit_w_derived(void) {
     ecs_world_t *world = ecs_init();
 
     ecs_entity_t q = ecs_entity_init(world, &(ecs_entity_desc_t){
-        .add = { EcsQuantity }
+        .add = ecs_ids( EcsQuantity )
     });
     test_assert(q != 0);
 
-    ecs_entity_t d = ecs_new_id(world);
+    ecs_entity_t d = ecs_new(world);
     test_assert(d != 0);
 
     ecs_set(world, d, EcsUnit, {
         .symbol = "d"
     });
 
-    ecs_entity_t u = ecs_new_id(world);
+    ecs_entity_t u = ecs_new(world);
     test_assert(u != 0);
 
     ecs_set(world, u, EcsUnit, {
@@ -736,25 +736,25 @@ void Units_set_unit_w_over(void) {
     ecs_world_t *world = ecs_init();
 
     ecs_entity_t q = ecs_entity_init(world, &(ecs_entity_desc_t){
-        .add = { EcsQuantity }
+        .add = ecs_ids( EcsQuantity )
     });
     test_assert(q != 0);
 
-    ecs_entity_t d = ecs_new_id(world);
+    ecs_entity_t d = ecs_new(world);
     test_assert(d != 0);
 
     ecs_set(world, d, EcsUnit, {
         .symbol = "d"
     });
 
-    ecs_entity_t o = ecs_new_id(world);
+    ecs_entity_t o = ecs_new(world);
     test_assert(o != 0);
 
     ecs_set(world, o, EcsUnit, {
         .symbol = "o"
     });
 
-    ecs_entity_t u = ecs_new_id(world);
+    ecs_entity_t u = ecs_new(world);
     test_assert(u != 0);
 
     ecs_set(world, u, EcsUnit, {
@@ -801,18 +801,18 @@ void Units_set_unit_w_prefix(void) {
     ecs_world_t *world = ecs_init();
 
     ecs_entity_t q = ecs_entity_init(world, &(ecs_entity_desc_t){
-        .add = { EcsQuantity }
+        .add = ecs_ids( EcsQuantity )
     });
     test_assert(q != 0);
 
-    ecs_entity_t d = ecs_new_id(world);
+    ecs_entity_t d = ecs_new(world);
     test_assert(d != 0);
 
     ecs_set(world, d, EcsUnit, {
         .symbol = "d"
     });
 
-    ecs_entity_t p = ecs_new_id(world);
+    ecs_entity_t p = ecs_new(world);
     test_assert(p != 0);
 
     ecs_set(world, p, EcsUnitPrefix, {
@@ -821,7 +821,7 @@ void Units_set_unit_w_prefix(void) {
         .translation.power = 2
     });
 
-    ecs_entity_t u = ecs_new_id(world);
+    ecs_entity_t u = ecs_new(world);
     test_assert(u != 0);
 
     ecs_set(world, u, EcsUnit, {
@@ -912,3 +912,13 @@ void Units_quantity_w_short_notation(void) {
 
     ecs_fini(world);
 }
+
+void Units_import_units_after_mini(void) {
+    ecs_world_t *world = ecs_mini();
+
+    ECS_IMPORT(world, FlecsUnits);
+    
+    test_assert(EcsMeters != 0);
+
+    ecs_fini(world);
+}
